#' Enrichment & Pathway Analysis UI Module
#' 
#' This module allows users to select a pathway analysis
#' database, set parameters for the analysis, and initiate the analysis process.
#' It supports visualization of the pathway analysis results and provides options
#' for downloading the results.
#' 
#' @param id A unique identifier for the UI module, necessary for distinguishing
#'           this module's input from others in the same Shiny application.
#' @return A tag list containing the UI elements required for the enrichment and
#'         pathway analysis module.
#' @export
enrichmentUI <- function(id) {
  ns <- NS(id)
  tagList(
    h2("Pathway and Enrichment Analysis"),
    fluidRow(
      column(width = 4,   
             h4("Select enrichment method"),
             selectInput(ns("analysisSelector"), "Select analysis", 
                         choices = c("GSEA: general interface for enrichment analysis",
                                     "GSEA: GO enrichment analysis",
                                     "GSEA: WikiPathways enrichment analysis",
                                     "GSEA: Reactome pathway enrichment analysis")),
             bsPopover(ns("analysisSelector"), "Choose the type of enrichment analysis you want to perform.", "Options include General Interface for Enrichment Analysis, GO Enrichment Analysis, WikiPathways Enrichment Analysis, and Reactome pathway enrichment analysis.", placement = "right", trigger = "hover", options = list(container = "body")),
             
      )
      
    ),
    uiOutput(ns("gmtSelectionUI")),
    uiOutput(ns("specificParameters"))
  )
}