library(data.table)
library(Seurat)
library(Matrix)

ch = fread(file = input$file$datapath, sep = ",", data.table = F, verbose = T) ## read the file using fread
gene_info = read.csv('R/Setup_Funtions/bin/gene_info.csv', header = T, row.names = 1) ## read gene info file for later use
gene_info$ID = rownames(gene_info)

total_cols = length(colnames(ch)) ## find total numbe rof columns in data

cells_list <- 2:total_cols ## create a cell-list 

chunk_size <- round(total_cols/10) # Define the number of elements you want in each chunk (10)

breakpts <- seq(0, length(cells_list)+2, by = chunk_size) # Generate breakpoints for cutting the vector into chunks starting from 0 

chunks <- cut(seq_along(cells_list), breaks = breakpts, labels = FALSE) # Cut the vector into chunks based on the breakpoints 

chunks <- split(cells_list, chunks) # Split the vector into chunks based on the cuts 

### Let's run a for loop extracting matrices and preparing a list of sparse matrices
sparse_mats = c()
for(f in 1:length(chunks))
{
	str = paste0('Executing Chunk', f)
	message(str)
	start_cell = chunks[[f]][1]
	length_chunk = length(chunks[[f]])
	end_cell = chunks[[f]][length_chunk]
	ext_c = c()
	ext_c = ch[,start_cell:end_cell]
	ext_c = as.matrix(ext_c)
	ext_c = Matrix(ext_c, sparse = T)
	sparse_mats = c(sparse_mats, ext_c)
}

final_sparse_mat = do.call(cbind, sparse_mats) ## cbind all sparse matrices
rownames(final_sparse_mat) = ch[,1] ## add rownames to sparse matrix
colnames(final_sparse_mat) = colnames(ch[,2:total_cols])

FOLDER_NAME = paste0('R/Setup_Funtions/bin/temp/', Sys.time(), '_LARGE_CSV_MM/')
dir.create(FOLDER_NAME) ## create a drirectory for resembling 10X out

mtx_file = paste0(FOLDER_NAME, "matrix.mtx")
gene_file = paste0(FOLDER_NAME, "genes.tsv")
barcodes_files = paste0(FOLDER_NAME, "barcodes.tsv")

writeMM(obj = final_sparse_mat, file = mtx_file) ## writing a Matrix object
write(x = rownames(final_sparse_mat), file = gene_file) ## writing genes.tsv
write(x = colnames(final_sparse_mat), file = barcodes_files) ## writing barcodes.tsv
gene_tsv = read.table(gene_file, sep = "\t", header = F) ## re-read genes.tsv
colnames(gene_tsv) = c("names") ## add column name
gene_info_ordered = c()
for(f in 1:length(colnames(gene_info)))
{
	check_value = match(gene_tsv$names, gene_info[,f])
	if(any(is.na(check_value)) == TRUE)
	{
		print ("match not found... try next column")
	}
	else
	{
		print ("Match found, ordering dataframe")
		gene_info_ordered = gene_info[match(gene_tsv$names, gene_info[,f]),]
	}
}
gene_tsv$gene_name = gene_info_ordered$Gene_Name ## add gene names for corresponding ens ids
gene_tsv_new = cbind(gene_tsv$gene_name, gene_tsv$gene_name) ## create a new data.frame with two gene name cols
write.table(gene_tsv_new, gene_file, sep = "\t", quote = F, col.names = F, row.names = F) ## write the new genes.tsv

seuratObject_raw = Read10X(FOLDER_NAME) ## read 10X out folder created
if(is.null(metadata) == FALSE)
{
	seuratObject_out = CreateSeuratObject(seuratObject_raw, project = input$projectName, min.cells = input$minCells, min.features = input$minFeatures, meta.data = metadata) ## create a seurat object with 10X based object
}
if(is.null(metadata) == TRUE)
{
	seuratObject_out = CreateSeuratObject(seuratObject_raw, project = input$projectName, min.cells = input$minCells, min.features = input$minFeatures) ## create a seurat object with 10X based object
}
unlink(FOLDER_NAME, recursive = TRUE)
rm(ch)
rm(chunks)
rm(sparse_mats)
rm(final_sparse_mat)
rm(seuratObject_raw)
gc()