#' DE Analysis UI Component
#'
#' This function creates a set of UI elements for data uploading in a Shiny app. It includes 
#' de analysis based on user selected control and treatment samples. 
#'
#' @param id A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return A UI element comprising a file input, a project name text input, and numeric inputs 
#'         for minimum cells and features. These elements are wrapped in a `tagList` for easy 
#'         integration into the app's UI.
#' @export
#'
#' @examples
#' In the UI definition of a Shiny app
#' deanalysisUI("DEModule")
#' 
#'       
deanalysisUI <- function(id) {
  ns <- NS(id)
  
  tagList(
    fluidRow(
      column(4,
        #h2('Differential Expression Analysis'),
        radioButtons(ns("sample_select"), label = "Sample Selection for DE Analysis", choices = c("Perform DE based on condition column in metadata", "Select Samples Manually for comparison"), selected = ''),
        conditionalPanel(
          condition = "input.sample_select == 'Select Samples Manually for comparison'",
          selectInput(ns("controls"), label = "Select Control/Wild-type Samples", choices = NULL, selected = NULL, multiple = TRUE),
          selectInput(ns("treatments"), label = "Select Treatment/Experiment Samples", choices = NULL, selected = NULL, multiple = TRUE),
          numericInput(ns("logFC_filter"), label = "Log2 Fold Change Cutoff", value = 0.5, min = 0, max = 100, step = 0.5),
          numericInput(ns("fdr_filter"), label = "False Discovery Rate (FDR) / Adjusted pValue Cutoff", value = 0.05, min = 0, max = 1, step = 0.01),
          downloadButton(ns('download_condition_table_manual'), 'Save Condition Table', style='color:#fff; background-color:#1c82e0'), ns =NS(id)),
        conditionalPanel(
          condition = "input.sample_select == 'Perform DE based on condition column in metadata'",
          DT::dataTableOutput(ns("condition_table")),
          numericInput(ns("logFC_filter_cond"), label = "Log2 Fold Change Cutoff", value = 0.5, min = 0, max = 100, step = 0.5),
          numericInput(ns("fdr_filter_cond"), label = "False Discovery Rate (FDR) / Adjusted pValue Cutoff", value = 0.05, min = 0, max = 1, step = 0.01), 
          selectInput(ns("controls_set"), label = "Set control samples", choices = NULL, selected = NULL, multiple = FALSE),
          selectInput(ns("treatment_set"), label = "Set treatment samples", choices = NULL, selected = NULL, multiple = FALSE), ns = NS(id)),
          actionButton(ns("performde"), label = "Perform DE Analysis", class = "btn-primary", icon = icon("upload"), style='color:#fff; background-color:#1c82e0'),
          conditionalPanel(
            condition = "input.performde > 0",
            selectInput(ns("coefficients"), label = "Select Coefficients", choices = NULL, selected = NULL, multiple = FALSE), ns = NS(id))),
      column(8,
        conditionalPanel(
          condition = "input.performde > 0",
          downloadButton(ns('downloadde_table'), 'Save Data', style='color:#fff; background-color:#1c82e0'),
          shinycssloaders::withSpinner(DT::dataTableOutput(ns("de_table")), size = 1), style = "height:500px; overflow-y: scroll;overflow-x: scroll;", ns = NS(id)))
    )
  )
}