#' QC UI Component
#'
#' This function creates a set of UI elements for data uploading in a Shiny app. It includes 
#' de analysis based on user selected control and treatment samples. 
#'
#' @param id A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return A UI element comprising a file input, a project name text input, and numeric inputs 
#'         for minimum cells and features. These elements are wrapped in a `tagList` for easy 
#'         integration into the app's UI.
#' @export
#'
#' @examples
#' In the UI definition of a Shiny app
#' qcUI("qcModule")
#' 
#'       
qcUI <- function(id) {
  ns <- NS(id)
  
  tagList(
    fluidRow(
      column(4,
        h2('QC Visualization'),
        radioButtons(ns("qcvisualization"), label = "Select Visualization Type", choices = c("PCA Plot", "Sample Distance Plot","MA Plot"), selected = NULL),
        conditionalPanel(
          condition = "input.qcvisualization == 'PCA Plot' || input.qcvisualization == 'Sample Distance Plot'",
          selectInput(ns("colorBy"), "Color by", choices = NULL, selected = NULL), ns = NS(id)),
        conditionalPanel(
          condition = "input.qcvisualization == 'PCA Plot'",
          actionButton(ns("performvqcis_pca"), label = "Generate Plot", class = "btn-primary", icon = icon("upload"), style='color:#fff; background-color:#1c82e0'), ns = NS(id)),
        conditionalPanel(
          condition = "input.qcvisualization == 'MA Plot'",
          actionButton(ns("performvqcis_ma"), label = "Generate Plot", class = "btn-primary", icon = icon("upload"), style='color:#fff; background-color:#1c82e0'), ns = NS(id)),
        conditionalPanel(
          condition = "input.qcvisualization == 'Sample Distance Plot'",
          actionButton(ns("performvqcis_sd"), label = "Generate Plot", class = "btn-primary", icon = icon("upload"), style='color:#fff; background-color:#1c82e0'), ns = NS(id))),
      column(8,
        conditionalPanel(
          condition = "input.performvqcis_pca > 0 && input.qcvisualization == 'PCA Plot'",
          downloadButton(ns('downloadPlot_pca'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
          shinycssloaders::withSpinner(plotOutput(ns("pca_plot"), width = "700", height = "600"), size = 1), ns = NS(id)),
        conditionalPanel(
          condition = "input.performvqcis_sd > 0 && input.qcvisualization == 'Sample Distance Plot'",
          downloadButton(ns('downloadPlot_sd'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
          shinycssloaders::withSpinner(plotOutput(ns("sd_plot"), width = "700", height = "600"), size = 1), ns = NS(id)),
        conditionalPanel(
          condition = "input.performvqcis_sd > 0 && input.qcvisualization == 'MA Plot'",
          downloadButton(ns('downloadPlot_ma'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
          shinycssloaders::withSpinner(plotOutput(ns("ma_plot"), width = "700", height = "600"), size = 1), ns = NS(id))
        )
    )
  )
}