#' Dimension Reduction Visualization UI
#' 
#' Generates plots (Dimension reduction plots) from user upload Seurat v5 object
#'
#' @param id  A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return A UI element containing action buttons for generating feature, dot, dim,
#'         violin, and ridge plots.
#' @export
#'
#' @examples
#' DiscoverydimredUI("DiscoverydimredModule")
DiscoverydimredUI <- function(id) {
  ns <- NS(id)
  
  tagList(
    fluidPage(
      tabsetPanel(type = "tabs",
                  tabPanel("Dim Plot",
                           br(),
                           fluidRow(
                             column(3, selectInput(ns("plotDimPlot_umap_options_group"), label = "Color by", choices = NULL, selected = NULL),
                                    actionButton(ns("plotDimPlot_umap"), "Non-Linear Dimension Reduction Plot", class = "btn-primary", style='color:#fff; background-color:#1c82e0')),
                             column(9,
                                    conditionalPanel(
                                      condition = "input.plotDimPlot_umap > 0",
                                      selectInput(ns("umapPlt"), "File Type Selection:", choices = list("PNG" = ".png", "PDF" = ".pdf", "SVG" = ".svg", "JPEG" = ".jpeg"), selected = ".png"),
                                      downloadButton(ns('downloadPlot_umap'), 'Save Plot', style='color:#fff; background-color:#1c82e0'), br(),
                                      shiny::plotOutput(ns("interactivePlot_umap"), width = "700", height = "600"), ns = NS(id))))),
                  
                  tabPanel("PCA",
                           br(),
                           fluidRow(
                             column(3, selectInput(ns("plotDimPlot_pca_options_group"), label = "Color PCA by", choices = NULL, selected = NULL),
                                    actionButton(ns("plotDimPlot_pca"), "PCA Plot", class = "btn-primary", style='color:#fff; background-color:#1c82e0')),
                             column(9,
                                    conditionalPanel(
                                      condition = "input.plotDimPlot_pca > 0",
                                      selectInput(ns("pcaPlt"), "File Type Selection:", choices = list("PNG" = ".png", "PDF" = ".pdf", "SVG" = ".svg", "JPEG" = ".jpeg"), selected = ".png"),
                                      downloadButton(ns('downloadPlot_pca'), 'Save Plot', style='color:#fff; background-color:#1c82e0'),
                                      shiny::plotOutput(ns("interactivePlot_pca"), width = "700", height = "600"), ns = NS(id))))),
                  
      )
    )
  )
}