#' Additional Plots UI
#' 
#' Generates the user interface elements for the additional plots module. This
#' includes generating specific types of plots (Feature Plot,
#' Dot Plot, Violin Plot, Ridge Plot).
#'
#' @param id  A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return A UI element containing action buttons for generating feature, dot, 
#'         violin, and ridge plots.
#' @export
#'
#' @examples
#' additionalPlotsUI("additionalPlotsModule")
#' 
additionalPlotsUI <- function(id) {
  ns <- NS(id)
  
  tagList(
    fluidPage(
      fluidRow(
        column(3, 
               radioButtons(ns("selection_type"), "Select type:", choices = list("Features/Gene" = "gene", "Metadata" = "metadata"), selected = "gene"),
               br(),
               selectizeInput(ns("plotFeaturePlot_options_features"), "Select features for visualization", choices = NULL, selected = NULL, multiple = TRUE),
               bsPopover(ns("plotFeaturePlot_options_features"), "Feature  visulaization", "Colors single cells on a plot according to selected genes. Multiple selections allowed.", placement = "right", trigger = "hover", options = list(container = "body"))
        )
      ),
      tabsetPanel(id = ns("Tabs"), type = "tabs",
                  tabPanel("Feature Plot",
                           br(),
                           fluidRow(
                             column(3,
                             ),
                             column(9,
                                    selectInput(ns("feature_genesPlt"), "File Type Selection:", choices = list("PNG" = ".png", "PDF" = ".pdf", "SVG" = ".svg", "JPEG" = ".jpeg"), selected = ".png"),
                                    downloadButton(ns('downloadPlot_feature_genes'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
                                    plotOutput(ns("interactivePlot_feature_genes"), width = "700", height = "600")
                             )
                           )
                  ),
                  tabPanel("Dot Plot",
                           br(),
                           fluidRow(
                             column(3, 
                                    selectInput(ns("plotDotPlot_options_group"), label = "Group dot plot by:", choices = NULL, selected = NULL),
                                    bsPopover(ns("plotDotPlot_options_group"), "Dot plot group selection", "Select group on the basis of which the cells should be grouped", placement = "right", trigger = "hover", options = list(container = "body"))
                             ),
                             column(9,
                                    selectInput(ns("featuredotPlt"), "File Type Selection:", choices = list("PNG" = ".png", "PDF" = ".pdf", "SVG" = ".svg", "JPEG" = ".jpeg"), selected = ".png"),
                                    downloadButton(ns('downloadPlot_dot'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
                                    plotOutput(ns("interactivePlot_dot"), width = "700", height = "600")
                             )
                           )
                  ),
                  tabPanel("Violin Plot",
                           br(),
                           fluidRow(
                             column(3, 
                                    selectInput(ns("plotViolinPlot_group"), label = "Group violin plot by:", choices = NULL, selected = NULL),
                                    bsPopover(ns("plotViolinPlot_group"), "Violin plot group selection", "Select group on the basis of which the cells should be grouped", placement = "right", trigger = "hover", options = list(container = "body"))
                             ),
                             column(9,
                                    selectInput(ns("VlnPlt"), "File Type Selection:", choices = list("PNG" = ".png", "PDF" = ".pdf", "SVG" = ".svg", "JPEG" = ".jpeg"), selected = ".png"),
                                    downloadButton(ns('downloadPlot_vln'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
                                    plotOutput(ns("interactivePlot_vln"), width = "700", height = "600")
                             )
                           )
                  ),
                  tabPanel("Ridge Plot",
                           br(),
                           fluidRow(
                             column(3, 
                                    selectInput(ns("plotRidgePlot_options_group"), label = "Group ridge plot by:", choices = NULL, selected = NULL),
                                    bsPopover(ns("plotRidgePlot_options_group"), "Ridge plot group selection", "Select group on the basis of which the cells should be grouped", placement = "right", trigger = "hover", options = list(container = "body"))
                             ),
                             column(9,
                                    selectInput(ns("RidgePlt"), "File Type Selection:", choices = list("PNG" = ".png", "PDF" = ".pdf", "SVG" = ".svg", "JPEG" = ".jpeg"), selected = ".png"),
                                    downloadButton(ns('downloadPlot_ridge'), 'Save plot', style='color:#fff; background-color:#1c82e0'),
                                    plotOutput(ns("interactivePlot_ridge"), width = "700", height = "600")
                             )
                           )
                  )
      )
    )
  )
}