#' Feature Selection UI Component
#'
#' This function generates a UI module for feature selection within a Shiny application,
#' particularly targeting the identification and visualization of highly variable features
#' in a dataset.
#' 
#' @param id A unique identifier for the UI module, necessary for distinguishing
#'           this module's input from others in the same Shiny application.
#'
#' @return UI elements for feature selection operations.
#' @export
#'
#' @examples
#' featureSelectionUI("featureSelectionModule")
featureSelectionUI <- function(id) {
  ns <- NS(id)
  tagList(
    
    # Button to trigger the identification of highly variable features.
    # Upon clicking, it should activate server-side logic to perform the analysis
    # and eventually update the variableFeaturePlot output with the visualization.
    actionButton(ns("openVariableFeaturesModal"), "Find Highly Variable Features", class = "btn-primary"),
    
    # The plot is expected to be generated by the server logic in response to
    # the action button click, illustrating the features identified as highly variable.
    shiny::plotOutput(ns("variableFeaturePlot"))
  )
}
