generateReport_Server <- function(id, gene_seuratObj, gene_seuratObj_processed, isoform_seuratObj_processed) {
  
  moduleServer(id, function(input, output, session) {
    ns <- session$ns

    output$scRNA_Report <- downloadHandler(
      filename = function() {
      paste0("CogentDS_v2.0_Analysis_Report", "-", Sys.Date(), ".html")
      },
      content = function(file) {
        withProgress(message = 'Preparing Report...', value = 0, {
          incProgress(0.3, detail = "Generating Report elements ...")
          tryCatch({
            preprocessed_gene_data = gene_seuratObj()
            processed_gene_data = gene_seuratObj_processed()
            processed_isoform_data = isoform_seuratObj_processed()
            Report_rmd <- paste0('R/Setup_Funtions/bin/', 'CogentDS_v2.0_scRNA_analyis_report_App.Rmd')
            params <- list(set_title = "Cogent™ NGS Discovery Software", seuratObject_gene_preprocessed = preprocessed_gene_data, seuratObject_gene_processed = processed_gene_data, seuratObject_isoform_processed = processed_isoform_data)
            rmarkdown::render(Report_rmd, output_file = file, params = params, envir = new.env(parent = globalenv()))
            shinyjs::disable("scRNA_Report")
            shinyalert('Analysis Report Downloaded', "Thank you! CogentDS Team", type = "success", confirmButtonCol = "#1c82e0")
          }, error = function(e) {
            shinyalert('Error in downloading the analysis report', 'Please check your input parameters', type = "error", confirmButtonCol = "#1c82e0")
          })
          incProgress(0.7, detail = "Completed")
        })
      } 
    )
  })
}