#' Pre-QC & QC UI Component
#'
#' Creates UI elements for pre-QC and QC operations within a Shiny application,
#' facilitating initial data quality assessment and subsequent quality control filtering.
#' This function generates action buttons that allow users to visualize data quality metrics
#' prior to QC and to perform QC based on set criteria and visualize post QC plots.
#'
#' @param id A unique identifier for the UI module, necessary for distinguishing
#'           this module's input from others in the same Shiny application. 
#'
#' @return A UI element comprising two action buttons: one for displaying pre-QC plots
#'         and another for performing QC operations.
#' @export
#'
#' @examples
#' # In the UI definition of a Shiny app
#' preQCQCUI("preQCQCModule")
preQCQCUI <- function(id) {
  ns <- NS(id)
  
  tagList(
    titlePanel("Pre-QC and QC Analysis"),
    uiOutput(ns("qcfiltersUI")),  # Place the QC filters first
    fluidRow(
      column(
        width = 12, 
        uiOutput(ns("plotTypeInput"))
      )
    ),
    fluidRow(
      column(
        width = 6, 
        uiOutput(ns("preQCPlots"))
      ),
      column(
        width = 6, 
        uiOutput(ns("postQCPlots"))
      )
    )
  )
}
