#' Upload Object UI Component
#'
#' This function allows user to upload a processed Seurat v5 object
#'
#' @param id A unique identifier for the UI module. This is necessary for distinguishing 
#'           this module's input from others in the same Shiny application.
#'
#' @return A UI element comprising seurat object. The element is wrapped in a `tagList` for easy 
#'         integration into the app's UI.
#' @export
#'
#' @examples
#' In the UI definition of a Shiny app
#' uploadObjectUI("uploadObjectModule")
#' 
#' @note The object the user uploads needs to have a processed seurat v5 structure

uploadObjectUI <- function(id) {
  ns <- NS(id)
  
  tagList(
    fluidRow(column(4,
      h2("Upload a CogentDS Processed .RDS File"),
      fileInput(ns("seurat_obj"), "Upload your processed CogentDS data", accept = c(".rds", ".rda")),
      actionButton(ns("uploadobject"), label = "Submit Upload", class = "btn-primary", icon = icon("upload"), style='color:#fff; background-color:#1c82e0')
      ),
      column(6,
        conditionalPanel(
        condition = "input.uploadobject > 0",
        actionButton(ns("reset_btn"), "Reset table", class = "btn-primary", style='color:#fff; background-color:#1c82e0'),
        DT::dataTableOutput(ns("seuratupload")), ns=NS(id))))
  )
}