####### load packages ##########
library(devtools)
library(shiny)
library(shinydashboard)
library(shinyjs)
library(Seurat)
library(SingleCellExperiment)
library(SingleR)
library(decontX)
library(celda)
library(plotly)
library(shinyalert)
library(dplyr)
library(presto)
library(shinycssloaders)
library(DT)
library(celldex)
library(data.table)
library(Matrix)
library(shinyBS)
library(org.Hs.eg.db)
library(org.Mm.eg.db)
library(enrichplot)
library(ggpubr)
library(rlist)
library(clusterProfiler)
library(tidyverse)
library(R.utils)
library(rmarkdown)
library(kableExtra)
library(Routliers)
library(ggrepel)
library(DESeq2)
library(MAST)
library(apeglm)
library(pheatmap)
library(EnhancedVolcano)
library(RColorBrewer)
library(ReactomePA)
library(msigdbr)
library(glmGamPoi)
##############################

options(Seurat.object.assay.version = "v5")

# Increase size limit for data
options(shiny.maxRequestSize=10000*1024^2) 

# Source the R scripts containing UI definitions and server logic for each module of the application
source('R/setup_functions.R')
##############################


# Define UI
ui <- dashboardPage(
  skin = "purple",
  dashboardHeader(title = 'Cogent™ NGS Discovery Software - scRNA', titleWidth = 400,
                            tags$li(a(href = 'https://www.takarabio.com/', img(src = "logo.png", title = "Company Home", height = "30px"), style = "padding-top:10px; padding-bottom:10px;"), class = "dropdown"),
                            tags$li(class = "dropdown", actionButton("refreshapp", "Refresh", icon = icon("refresh"), style='color: black; background-color: white; border: transparent'))),
  dashboardSidebar(
    width = 150,
    sidebarMenuOutput("CogentDSsidebarmenu")
  ),

  dashboardBody(
    shinyjs::useShinyjs(),
    tags$head(tags$style(".sidebar-menu li { margin-bottom: 0px; margin-top: 0px;}")),
    tags$head(tags$style(".sidebar-menu li { padding: 0px 0px 0px 0px; display: block}")),
    tags$head(tags$style(HTML('

        .wrapper {height: auto !important; position:relative; overflow-x:hidden; overflow-y:hidden}

         .skin-purple .main-header .logo {
          background-color: white;
          color: black;
          border-bottom-style: solid;
          text-align: left;
          padding: 0px 0px 0px 10px;
          font-style: normal;
          font-family: math;
        }

        .skin-purple .main-header .navbar{
          background-color: white;
          border-bottom: #e4e4e4 solid;
        }

        .skin-purple .main-header .navbar .sidebar-toggle {
          background-color: white;
          color: black;
        }

        /* main sidebar */
          .skin-blue .main-sidebar {
            background-color: #e4e4e4;
        }

        .skin-purple .left-side, .skin-purple .main-sidebar, .skin-purple .wrapper {
          background-color: #e4e4e4;
          border-left-color: #e4e4e4;
        }

        .skin-purple .sidebar-menu>li.active>a, .skin-purple .sidebar-menu>li:hover>a {
          color: #1c82e0;
          text-align: left;
          background-color: #e4e4e4;
          border-left-color: #e4e4e4;
        }

        .skin-purple .main-header .navbar .sidebar-toggle:hover {
          background-color: white
        }

        .skin-purple .main-header .logo:hover {
          background-color: white
        }

        .skin-purple .sidebar a {
          color: darkgreen;
        }

        /* body */
          .content-wrapper, .right-side {
          background-color: white;
        }
        
    '))),
    tabItems(
      tabItem(tabName = "landingUI", landingUI("landingUIModule")),
      tabItem(tabName = "dataUpload", dataUploadUI("dataUploadModule"), actionButton("gotodecontx", "Next: Perform Ambient RNA Filtering", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "Decontamination", decontaminationUI("DecontaminationModule"), br(), br(), br(), actionButton("gotoqc", "Next: Perform QC", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "preQCQC", preQCQCUI("preQCQCModule"), actionButton("gotonorm", "Next: Perform Normalization, Feature Selection & Scaling", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "normalizationScaling", normalizationScalingUI("normalizationScalingModule"), actionButton("gotopca", "Next: Perform PCA Analysis", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "dimReduction", dimReductionUI("dimReductionModule"), actionButton("gotoclust", "Next: Perform Clustering", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "clusteringVisualization", clusteringVisualizationUI("clusteringVisualizationModule"), actionButton("gotoanno", "Next: Perform Cell Type Annotation", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "cellAnnotations", cellTypeAnnotationUI("cellTypeAnnotationModule"), actionButton("gotoCustomSel", "Next: Perform Custom Lasso Selection", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "CustomSel", umapLassoSelectionUI("umapLassoSelectionModule"), actionButton("gotodiff", "Next: Perform Differential Gene Expression Analysis", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "diffExpression", diffExpressionUI("differentialExpressionModule"), actionButton("gotopathways", "Next: Perform Pathway Analysis", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "PathwayAnalysis", enrichmentUI("PathwayAnalysisModule"), actionButton("gotoplots", "Next: Generate Additonal Plots", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "additionalPlots",  additionalPlotsUI("additionalPlotsModule"), actionButton("gotoisoform", "Next: Isoform Analysis", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "isoformanalysis",  isoformUI("isoformModule"), actionButton("gotoFusion", "Next: Gene Fusion Analysis", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "GeneFusion", geneFusionUI("geneFusionModule"), actionButton("gotoClonotype", "Next: Clonotype Analysis", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "Clonotype", clonotypeAnalysisUI("clonotypeAnalysisModule"), actionButton("gotoreport", "Next: Generate HTML Report", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "generateReport", generateReport_UI("generateReport_Module"), actionButton("gotodownload", "Next: Download Processed Data", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "downloadseurat_v5", downloadSeurat_v5_UI("DownloadSeurat_Module"), actionButton("gotoMain", "Go To Main Page", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "UploadObject", uploadObjectUI("uploadObjectModule"), actionButton("gotoDimred", "Next: Dimension Reduction Visualization", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "DiscoveryDimred", DiscoverydimredUI("DiscoverydimredModule"), actionButton("gotovisualobj", "Next: Data Visualization", class = "btn-primary", style = "position: absolute; bottom: 0; right: 0; color:#fff; background-color:#1c82e0")),
      tabItem(tabName = "VisualizationObject", visualizationObjectUI("VisualizationObjectModule"))
    )
  )
)

# Server Logic
server <- function(input, output, session) {
  
  ns <- session$ns

  ########## Load Server functions

  

  output$workflow <- renderImage({
  list(src = 'www/workflow_new.png',
         contentType = 'image/png',
         alt = "This is alternate text", width = 600, height = 750)
  }, deleteFile = F)


  output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
  sidebarMenu(id='CogentDSsidebarmenu',
              menuItem("scRNA", tabName = "landingUI", icon = icon("dashboard"), selected = "landingUI"))

  })

  observeEvent(input$refreshapp, {
    shinyjs::refresh()
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu',
          menuItem("scRNA", tabName = "landingUI", icon = icon("dashboard")))

    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "landingUI")
  })

  observeEvent(input$confirm, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu',
                menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload"), selected = "dataUpload"))    
    })
    
  })

  observeEvent(input$confirm1, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu',
                menuItem("Upload Data", tabName = "UploadObject", icon = icon("upload")))          
    })
  })

  observeEvent(input$gotodecontx, {
     output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
        menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
        menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")))
      })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "Decontamination")
  })

  observeEvent(input$gotoqc, {
     output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
        menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
        menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
        menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")))
      })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "preQCQC")
  })

  observeEvent(input$gotonorm, {
     output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
        menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
        menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
        menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
        menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")))
      })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "normalizationScaling")
  })

   observeEvent(input$gotopca, {
     output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
        menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
        menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
        menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
        menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
        menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")))
      })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "dimReduction")
  })

  observeEvent(input$gotoclust, {
     output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
        menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
        menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
        menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
        menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
        menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
        menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")))
      })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "clusteringVisualization")
  })

  observeEvent(input$gotoanno, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")))
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "cellAnnotations")
  })
  
  
  observeEvent(input$gotoCustomSel, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")))
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "CustomSel")
  })
  
  observeEvent(input$gotodiff, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")),
                  menuItem("Find Markers", tabName = "diffExpression", icon = icon("chart-line")))
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "diffExpression")
  })
  
  
  observeEvent(input$gotopathways, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")),
                  menuItem("Find Markers", tabName = "diffExpression", icon = icon("chart-line")),
                  menuItem("Pathway Analysis", tabName = "PathwayAnalysis", icon = icon("chart-line")))
      
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "PathwayAnalysis")
  })
  
  
  observeEvent(input$gotoplots, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")),
                  menuItem("Find Markers", tabName = "diffExpression", icon = icon("chart-line")),
                  menuItem("Pathway Analysis", tabName = "PathwayAnalysis", icon = icon("chart-line")),
                  menuItem("Additional Viz", tabName = "additionalPlots", icon = icon("chart-area")))
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "additionalPlots")
  })
  

  observeEvent(input$gotoisoform, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")),
                  menuItem("Find Markers", tabName = "diffExpression", icon = icon("chart-line")),
                  menuItem("Pathway Analysis", tabName = "PathwayAnalysis", icon = icon("chart-line")),
                  menuItem("Additional Viz", tabName = "additionalPlots", icon = icon("chart-area")),
                  menuItem("Isoform Analysis", tabName = "isoformanalysis", icon = icon("chart-area")))
                  
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "isoformanalysis")
  })

  observeEvent(input$gotoFusion, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")),
                  menuItem("Find Markers", tabName = "diffExpression", icon = icon("chart-line")),
                  menuItem("Pathway Analysis", tabName = "PathwayAnalysis", icon = icon("chart-line")),
                  menuItem("Additional Viz", tabName = "additionalPlots", icon = icon("chart-area")),
                  menuItem("Isoform Analysis", tabName = "isoformanalysis", icon = icon("chart-area")),
                  menuItem("Fusion Analysis", tabName = "GeneFusion", icon = icon("chart-area")))
                  
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "GeneFusion")
  })
  
  observeEvent(input$gotoClonotype, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")),
                  menuItem("Find Markers", tabName = "diffExpression", icon = icon("chart-line")),
                  menuItem("Pathway Analysis", tabName = "PathwayAnalysis", icon = icon("chart-line")),
                  menuItem("Additional Viz", tabName = "additionalPlots", icon = icon("chart-area")),
                  menuItem("Isoform Analysis", tabName = "isoformanalysis", icon = icon("chart-area")),
                  menuItem("Fusion Analysis", tabName = "GeneFusion", icon = icon("chart-area")),
                  menuItem("Clonotype Analysis", tabName = "Clonotype", icon = icon("chart-area")))
      
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "Clonotype")
  })
  
  observeEvent(input$gotoreport, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")),
                  menuItem("Find Markers", tabName = "diffExpression", icon = icon("chart-line")),
                  menuItem("Pathway Analysis", tabName = "PathwayAnalysis", icon = icon("chart-line")),
                  menuItem("Additional Viz", tabName = "additionalPlots", icon = icon("chart-area")),
                  menuItem("Isoform Analysis", tabName = "isoformanalysis", icon = icon("chart-area")),
                  menuItem("Fusion Analysis", tabName = "GeneFusion", icon = icon("chart-area")),
                  menuItem("Clonotype Analysis", tabName = "Clonotype", icon = icon("chart-area")),
                  menuItem("Analysis Report", tabName = "generateReport", icon = icon("download")))
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "generateReport")
  })

  observeEvent(input$gotodownload, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "dataUpload", icon = icon("upload")),
                  menuItem("Ambient RNA", tabName = "Decontamination", icon = icon("flask")),
                  menuItem("QC", tabName = "preQCQC", icon = icon("tachometer-alt")),
                  menuItem("Normalization", tabName = "normalizationScaling", icon = icon("balance-scale")),
                  menuItem("PCA", tabName = "dimReduction", icon = icon("compress-arrows-alt")),
                  menuItem("Clustering", tabName = "clusteringVisualization", icon = icon("project-diagram")),
                  menuItem("Annotate Cells", tabName = "cellAnnotations", icon = icon("microscope")),
                  menuItem("Custom Selection", tabName = "CustomSel", icon = icon("chart-line")),
                  menuItem("Find Markers", tabName = "diffExpression", icon = icon("chart-line")),
                  menuItem("Pathway Analysis", tabName = "PathwayAnalysis", icon = icon("chart-line")),
                  menuItem("Additional Viz", tabName = "additionalPlots", icon = icon("chart-area")),
                  menuItem("Isoform Analysis", tabName = "isoformanalysis", icon = icon("chart-area")),
                  menuItem("Fusion Analysis", tabName = "GeneFusion", icon = icon("chart-area")),
                  menuItem("Clonotype Analysis", tabName = "Clonotype", icon = icon("chart-area")),
                  menuItem("Analysis Report", tabName = "generateReport", icon = icon("download")),
                  menuItem("Download Object", tabName = "downloadseurat_v5", icon = icon("download")))
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "downloadseurat_v5")
  })

  observeEvent(input$gotoMain, {
    shinyjs::refresh()
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu',
          menuItem("scRNA", tabName = "landingUI", icon = icon("dashboard")))

    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "landingUI")
  })
  
  observeEvent(input$gotoDimred, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "UploadObject", icon = icon("upload")),
                  menuItem("PCA/UMAP/tSNE", tabName = "DiscoveryDimred", icon = icon("chart-area")))
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "DiscoveryDimred")
  })
  
  observeEvent(input$gotovisualobj, {
    output$CogentDSsidebarmenu <- shinydashboard::renderMenu({
      sidebarMenu(id='CogentDSsidebarmenu', 
                  menuItem("Upload Data", tabName = "UploadObject", icon = icon("upload")),
                  menuItem("PCA/UMAP/tSNE", tabName = "DiscoveryDimred", icon = icon("chart-area")),
                  menuItem("Expression", tabName = "VisualizationObject", icon = icon("microscope")))
      
  
    })
    updateTabItems(session, "CogentDSsidebarmenu", selected = "VisualizationObject")
  })
  
  
  # Reactive to choose DE results based on user input
  deResults <- reactive({
    if (input$deResultChoice == "diffExpression") {
      return(diffExpressionResults)
    } else if (input$deResultChoice == "lassoSelection") {
      return(lassoSelectionResults)
    }
  })

  set.seed(123456) #### setting seed
  seuratObject <- dataUploadServer("dataUploadModule")
  seuratObject_decontX <- decontaminationServer("DecontaminationModule", seuratObject)
  seuratObjectQC <- preQCQCServer("preQCQCModule", seuratObject_decontX)
  normalizationScalingServer("normalizationScalingModule", seuratObjectQC)
  dimReductionServer("dimReductionModule", seuratObjectQC)
  clusteringVisualizationServer("clusteringVisualizationModule", seuratObjectQC)
  cellTypeAnnotationServer("cellTypeAnnotationModule", seuratObjectQC)
  umapLassoSelectionServer("umapLassoSelectionModule", seuratObjectQC)
  deResults <- diffExpressionServer("differentialExpressionModule", seuratObjectQC)
  enrichmentServer("PathwayAnalysisModule", deResults)
  additionalPlotsServer("additionalPlotsModule", seuratObjectQC)
  isoformObject <- isoformServer("isoformModule", seuratObjectQC)
  geneFusionServer("geneFusionModule", seuratObjectQC)
  clonotypeAnalysisServer("clonotypeAnalysisModule", seuratObjectQC)
  generateReport_Server("generateReport_Module", seuratObject, seuratObjectQC, isoformObject)
  downloadSeurat_v5_Server("DownloadSeurat_Module", seuratObjectQC)
  seuratObject_processed <- uploadObjectServer("uploadObjectModule")
  visualizationObjectServer("VisualizationObjectModule", seuratObject_processed)
  DiscoverydimServer("DiscoverydimredModule", seuratObject_processed)
  
}

# Run the application
shinyApp(ui = ui, server = server)
